import {
  Button,
  Card,
  Col,
  Form,
  Result,
  Row,
  Space,
  Spin,
  Switch,
} from "antd";
import { useTranslation } from "react-i18next";
import {
  Constants,
  EncodeStringToBase64,
  getUserSessionFromLocalStorage,
  myFetch,
} from "../../../utils";
import { useEffect, useRef, useState } from "react";
import MyCenteredContainer from "../../../Components/MyContainer";
import {
  MyCalendarMaxFutureBookingDaysFormInput,
  MyCalendarMinEarliestBookingTimeFormInput,
  MyPasswordFormInput,
} from "../../../Components/MyFormInputs";
import {
  RequestState,
  RequestStateItem,
} from "../../../Components/MyRequestStateItem";
import MyModal, {
  MyModalCloseConfirmButtonFooter,
} from "../../../Components/MyModal";
import { useParams } from "react-router-dom";

export default function StoreCalendar() {
  const { t } = useTranslation();
  const { storeId } = useParams();

  const [calendarSettings, setCalendarSettings] = useState({});
  const [isRequesting, setIsRequesting] = useState(true);

  useEffect(() => {
    // delete session cookie
    document.cookie = `session=; path=/; expires=Thu, 01 Jan 1970 00:00:01 GMT;`;

    myFetch("/calendar/settings", "GET")
      .then((res) => {
        setIsRequesting(false);
        setCalendarSettings(res);
      })
      .catch((err) => {
        console.log(err);
      });
  }, []);

  if (isRequesting) {
    return (
      <MyCenteredContainer>
        <Spin size="large" />
      </MyCenteredContainer>
    );
  }

  return (
    <>
      {calendarSettings.connected === false ? (
        <MyCenteredContainer>
          <Result
            status="warning"
            title={t("calendar.googleCalendarNotConnected.title")}
            subTitle={t("calendar.googleCalendarNotConnected.subTitle")}
            extra={
              <Button
                type="primary"
                onClick={() => {
                  document.cookie = `session=${getUserSessionFromLocalStorage()}; path=/;`;

                  window.location.href = `${Constants.API_ADDRESS}/calendar/auth/google`;
                }}
              >
                {t("calendar.googleCalendarNotConnected.button")}
              </Button>
            }
          />
        </MyCenteredContainer>
      ) : (
        <>
          <h1>{t("calendar.pageTitle")}</h1>

          <Row gutter={[16, 16]}>
            <Col xs={24} md={12}>
              <CardPersonalCalendarSettings
                settings={calendarSettings.userSettings}
              />
            </Col>
            <Col xs={24} md={12}>
              {calendarSettings.storeSettings && (
                <CardStoreCalendarSettings
                  settings={calendarSettings.storeSettings}
                />
              )}
            </Col>

            <Col xs={24}>
              <Card title={t("calendar.calendarFrameCustomerView")}>
                <CalendarFrame storeId={storeId} />
              </Card>
            </Col>
          </Row>
        </>
      )}
    </>
  );
}

function CalendarFrame({ storeId }) {
  const [isLoading, setIsLoading] = useState(true);

  return (
    <>
      {isLoading && (
        <div
          style={{
            display: "flex",
            height: "50vh",
            justifyContent: "center",
            alignItems: "center",
          }}
        >
          <Spin size="large" />
        </div>
      )}
      <iframe
        onLoad={() => setIsLoading(false)}
        style={{ border: 0, borderRadius: 12 }}
        width="100%"
        height={530}
        src={`${Constants.EMBED_CALENDAR_ADDRESS}${storeId}`}
      />
    </>
  );
}

function CardPersonalCalendarSettings({ settings }) {
  const { t } = useTranslation();
  const [form] = Form.useForm();
  const [formUnlinkCalendar] = Form.useForm();

  const [requestState, setRequestState] = useState(RequestState.INIT);
  const delayTimeout = useRef();
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [isRequesting, setIsRequesting] = useState(false);

  const usingPrimaryCalendar = Form.useWatch(
    "calendarUsingPrimaryCalendar",
    form
  );

  const handleModalClose = () => {
    setIsModalOpen(false);
  };

  useEffect(() => {
    if (!settings) return;

    form.setFieldValue(
      "calendarUsingPrimaryCalendar",
      settings.calendar_using_primary_calendar
    );
  }, []);

  useEffect(() => {
    if (usingPrimaryCalendar === undefined) return;

    if (RequestState.INIT === requestState) {
      setRequestState(RequestState.NOTHING);
      return;
    }

    setRequestState(RequestState.REQUESTING);

    if (delayTimeout.current) {
      clearTimeout(delayTimeout.current);
    }

    delayTimeout.current = setTimeout(() => {
      myFetch("/calendar/settings/personal", "POST", {
        calendarUsingPrimaryCalendar: usingPrimaryCalendar,
      })
        .then(() => setRequestState(RequestState.SUCCESS))
        .catch((errStatus) => {
          console.log(errStatus);
          setRequestState(RequestState.FAILED);
        });
    }, 500);
  }, [usingPrimaryCalendar]);

  useEffect(() => {
    if (!isModalOpen) return;

    formUnlinkCalendar.resetFields();
  }, [isModalOpen]);

  return (
    <>
      <Card
        title={t("calendar.cardPersonalCalendarSettings.title")}
        extra={
          <Space>
            <RequestStateItem
              state={requestState}
              setRequestState={setRequestState}
            />

            <Button danger onClick={() => setIsModalOpen(true)}>
              {t("calendar.unlinkGoogleCalendar.button")}
            </Button>
          </Space>
        }
      >
        <Form form={form} requiredMark={false}>
          <Form.Item
            name="calendarUsingPrimaryCalendar"
            label={t("common.calendarUsingPrimaryCalendar")}
            valuePropName="checked"
          >
            <Switch />
          </Form.Item>
        </Form>
      </Card>

      <MyModal
        title={t("calendar.unlinkGoogleCalendar.title")}
        isOpen={isModalOpen}
        onCancel={handleModalClose}
        footer={
          <MyModalCloseConfirmButtonFooter
            onCancel={handleModalClose}
            isConfirmButtonLoading={isRequesting}
            onConfirm={() => {
              formUnlinkCalendar.validateFields().then((values) => {
                setIsRequesting(true);

                myFetch("/calendar/settings/personal/unlink", "POST", {
                  password: EncodeStringToBase64(values.password),
                })
                  .then(() => window.location.reload())
                  .catch((errStatus) => {
                    console.log(errStatus);
                    setIsRequesting(false);
                    // TODO: show error message
                  });
              });
            }}
          />
        }
      >
        <>
          <Form
            form={formUnlinkCalendar}
            requiredMark={false}
            layout="vertical"
          >
            <p>{t("calendar.unlinkGoogleCalendar.description")}</p>
            <MyPasswordFormInput />
          </Form>
        </>
      </MyModal>
    </>
  );
}

function CardStoreCalendarSettings({ settings }) {
  const { t } = useTranslation();
  const [form] = Form.useForm();

  const [requestState, setRequestState] = useState(RequestState.INIT);
  const delayTimeout = useRef();

  const calendarMaxFutureBookingDays = Form.useWatch(
    "calendarMaxFutureBookingDays",
    form
  );

  const calendarMinEarliestBookingTime = Form.useWatch(
    "calendarMinEarliestBookingTime",
    form
  );

  useEffect(() => {
    if (!settings) return;

    form.setFieldsValue({
      calendarMaxFutureBookingDays: settings.calendar_max_future_booking_days,
      calendarMinEarliestBookingTime:
        settings.calendar_min_earliest_booking_time,
    });
  }, []);

  useEffect(() => {
    // undefined on first render
    // null when input is empty
    if (
      calendarMaxFutureBookingDays === undefined ||
      calendarMinEarliestBookingTime === undefined ||
      calendarMaxFutureBookingDays === null ||
      calendarMinEarliestBookingTime === null
    )
      return;

    if (RequestState.INIT === requestState) {
      setRequestState(RequestState.NOTHING);
      return;
    }

    setRequestState(RequestState.REQUESTING);

    if (delayTimeout.current) {
      clearTimeout(delayTimeout.current);
    }

    delayTimeout.current = setTimeout(() => {
      myFetch("/calendar/settings/store", "POST", {
        calendarMaxFutureBookingDays,
        calendarMinEarliestBookingTime,
      })
        .then(() => setRequestState(RequestState.SUCCESS))
        .catch((errStatus) => {
          console.log(errStatus);
          setRequestState(RequestState.FAILED);
        });
    }, 500);
  }, [calendarMaxFutureBookingDays, calendarMinEarliestBookingTime]);

  return (
    <Card
      title={t("calendar.cardStoreCalendarSettings.title")}
      extra={
        <RequestStateItem
          state={requestState}
          setRequestState={setRequestState}
        />
      }
    >
      <Form form={form} requiredMark={false}>
        <>
          <MyCalendarMaxFutureBookingDaysFormInput />

          <MyCalendarMinEarliestBookingTimeFormInput />
        </>
      </Form>
    </Card>
  );
}
